// This class was created by JaTiTV.

package net.t2code.t2codelib.VELOCITY.system;

import com.google.inject.Inject;
import com.velocitypowered.api.event.Subscribe;
import com.velocitypowered.api.event.proxy.ProxyInitializeEvent;
import com.velocitypowered.api.plugin.PluginContainer;
import com.velocitypowered.api.plugin.annotation.DataDirectory;
import com.velocitypowered.api.proxy.ProxyServer;
import com.velocitypowered.api.proxy.messages.MinecraftChannelIdentifier;
import lombok.Getter;

import net.t2code.t2codelib.util.T2C_GenerateFrame;
import net.t2code.t2codelib.util.T2C_PlatformDetector;
import net.t2code.t2codelib.util.T2C_Util;
import net.t2code.t2codelib.VELOCITY.api.pluginMessaging.T2CV_PluginMessageProxyCommand;
import net.t2code.t2codelib.VELOCITY.api.pluginMessaging.autoResponse.T2CV_API_AutoResponse;
import net.t2code.t2codelib.VELOCITY.api.pluginMessaging.commandgui.T2CV_API_CGUI;
import net.t2code.t2codelib.VELOCITY.api.pluginMessaging.opSecurity.T2CV_API_OpSecurity;
import net.t2code.t2codelib.VELOCITY.api.proxyPlayers.T2CV_Players;
import net.t2code.t2codelib.VELOCITY.api.update.T2CV_UpdateAPI;
import net.t2code.t2codelib.VELOCITY.system.bstats.T2CV_Metrics;
import net.t2code.t2codelib.VELOCITY.system.config.T2CV_LibConfig;
import org.slf4j.Logger;

import java.io.IOException;
import java.nio.file.Path;

public class T2CV_Main {

    @Getter
    private static ProxyServer server;
    @Getter
    private static Logger logger;
    @Getter
    private PluginContainer plugin;
    @Getter
    private final long starttime;
    @Getter
    private static Path dataDirectory;

    private final T2CV_Metrics.Factory metricsFactory;

    public static final MinecraftChannelIdentifier bcmd = MinecraftChannelIdentifier.from(T2C_Util.getPluginChannel_ProxyCommand());
    public static final MinecraftChannelIdentifier bonlp = MinecraftChannelIdentifier.from(T2C_Util.getPluginChannel_ProxyOnlinePlayers());
    public static final MinecraftChannelIdentifier cguiopl = MinecraftChannelIdentifier.from(T2C_Util.getPluginChannel_CGUIOnlinePlayers());
    public static final MinecraftChannelIdentifier aresp = MinecraftChannelIdentifier.from(T2C_Util.getPluginChannel_AutoResponse());
    public static final MinecraftChannelIdentifier opsec = MinecraftChannelIdentifier.from(T2C_Util.getPluginChannel_OPSecurity());


    @Inject
    public T2CV_Main(ProxyServer server, Logger logger, @DataDirectory Path dataDirectory, T2CV_Metrics.Factory metricsFactory) {
        T2C_PlatformDetector.scan();
        starttime = System.currentTimeMillis();
        T2CV_Main.server = server;
        T2CV_Main.logger = logger;
        T2CV_Main.dataDirectory = dataDirectory;
        this.metricsFactory = metricsFactory;
    }

    @Subscribe
    public void onLoad(ProxyInitializeEvent e) {
        new pl(this, server);
        //   logger.info("║");
        for (String s : T2C_Util.getLoadLogo()) {
            logger.info(T2C_GenerateFrame.removeColorCodes(s));
        }
        //    Thread.sleep(5000);
        logger.info(T2C_GenerateFrame.setCenterAligned("", "Author: " + String.valueOf(plugin.getDescription().getAuthors()).replace("[", "").replace("]", "")));
        logger.info(T2C_GenerateFrame.setCenterAligned("", "Version: " + plugin.getDescription().getVersion().orElse("unknown")));
        logger.info(T2C_GenerateFrame.setCenterAligned("", "Spigot:  " + T2C_Util.getSpigot()));
        logger.info(T2C_GenerateFrame.setCenterAligned("", "Discord: " + T2C_Util.getDiscord()));


        try {
            T2CV_LibConfig.set(dataDirectory, logger);
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }

        //   new T2CVcmd(server, logger);
        //   server.getEventManager().register(this, new T2CVpluginMessagingCmd(server, logger));

        //API

        server.getEventManager().register(this, new T2CV_PluginMessageProxyCommand(server, logger));


        server.getChannelRegistrar().register(T2CV_Main.bonlp);
        T2CV_Players.sendToSpigotDeleteAll();

        if ((boolean) T2CV_LibConfig.VALUES.apiCommandGUIEnable.getValue()) {
            server.getEventManager().register(this, new T2CV_API_CGUI(server, logger));
            T2CV_API_CGUI.sendToSpigotDeleteAll();
        }

        if ((boolean) T2CV_LibConfig.VALUES.apiAutoResponse.getValue()) {
            server.getEventManager().register(this, new T2CV_API_AutoResponse(server, logger));
        }

        if ((boolean) T2CV_LibConfig.VALUES.apiOpSecurity.getValue()) {
            server.getEventManager().register(this, new T2CV_API_OpSecurity(server, logger));
        }


        T2CV_Metrics.bStats(this, T2C_Util.getBstatsID(), metricsFactory);
        T2CV_UpdateAPI.onUpdateCheckTimer(logger, plugin, server, T2C_Util.getVPrefix(), T2C_Util.getDiscord(), T2C_Util.getSpigotID(), T2C_Util.getGit());
        logger.info(T2C_GenerateFrame.setCenterAligned("", ""));
        logger.info(T2C_GenerateFrame.setCenterAligned("", "Plugin loaded successfully - " + (System.currentTimeMillis() - starttime) + "ms"));

        logger.info(T2C_GenerateFrame.getBOTTOM_BORDER());
    }


    public class pl {
        public pl(Object pl, ProxyServer server) {
            plugin = server.getPluginManager().fromInstance(pl)
                    .orElseThrow(() -> new IllegalArgumentException("The provided instance is not a plugin"));
        }
    }
}
